///////////////////////////////////////////////////
//                        //
///////////////////////////////////////////////////

#include <string.h>
#include "FileIO.h"


//---------------------------------------------------------------
//  
FileIO::TFile::~TFile()
{
    delete[] O2M_M_FName;
    if (O2M_M_f) fclose(O2M_M_f);
}


//---------------------------------------------------------------
//        
int FileIO::TFile::IOResult()
{
    int res = O2M_M_IORes;
    O2M_M_IORes = 0;
    return res;
}


//---------------------------------------------------------------
//    
void FileIO::TFile::Assign(const char *FileName, const int FileNameSize)
{
    delete[] O2M_M_FName;
    O2M_M_FName = NULL;
    if (!FileName) return;

    O2M_M_FName = new char[FileNameSize];
    strncpy(O2M_M_FName, FileName, FileNameSize - 1);
    O2M_M_FName[FileNameSize - 1] = '\0';
}


//---------------------------------------------------------------
//   
void FileIO::TFile::Reset()
{
    if (O2M_M_IORes) return;
    if (O2M_M_f) {
        O2M_M_IORes = AlreadyOpened;
        return;
    }
    O2M_M_f = fopen(O2M_M_FName, "r");
    if (!O2M_M_f) O2M_M_IORes = ResetError;
    else O2M_M_OpenState = 'r';
}


//---------------------------------------------------------------
//   
void FileIO::TFile::Rewrite()
{
    if (O2M_M_IORes) return;
    if (O2M_M_f) {
        O2M_M_IORes = AlreadyOpened;
        return;
    }
    O2M_M_f = fopen(O2M_M_FName, "w");
    if (!O2M_M_f) O2M_M_IORes = RewriteError;
    else O2M_M_OpenState = 'w';
}

//---------------------------------------------------------------
//    
int FileIO::TFile::Write(char *Block, const int BlockSize)
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'w') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    return fwrite(Block, 1, BlockSize - 1, O2M_M_f);
}


//---------------------------------------------------------------
//         
int FileIO::TFile::Writeln(char *Block, const int BlockSize)
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'w') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    int res = Write(Block, BlockSize);
    return res + fprintf(O2M_M_f, "\n");
}


//---------------------------------------------------------------
//    
int FileIO::TFile::Read(char *Block, const int BlockSize)
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'r') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    int res = fread(Block, 1, BlockSize - 1, O2M_M_f);
    Block[res] = '\0';    // 0    
    return res;
}


//---------------------------------------------------------------
//  
bool FileIO::TFile::Eof()
{
    if (O2M_M_f) return feof(O2M_M_f) != 0;
    else return true;
}


//---------------------------------------------------------------
//  
void FileIO::TFile::Close()
{
    if (!O2M_M_f) {
        O2M_M_IORes = AlreadyClosed;
        return;
    }
    if (fclose(O2M_M_f)) O2M_M_IORes = CloseError;
    else O2M_M_f = NULL;
    O2M_M_OpenState = 'n';
}

//---------------------------------------------------------------
//     
int FileIO::TFile::ReadChar(char &ch)
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'r') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    fscanf(O2M_M_f, "%c", &ch);
    return 1;
}

//---------------------------------------------------------------
//     
int FileIO::TFile::ReadInt(int &i)
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'r') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    fscanf(O2M_M_f, "%d", &i);
    return 1;
}

//---------------------------------------------------------------
//      
int FileIO::TFile::ReadLongInt(long &i)
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'r') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    fscanf(O2M_M_f, "%d", &i);
    return 1;
}

//---------------------------------------------------------------
//     
int FileIO::TFile::ReadReal(double &x)
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'r') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    float f;
    fscanf(O2M_M_f, "%e", &f);
    x = f;
    return 1;
}

//---------------------------------------------------------------
//      
int FileIO::TFile::ReadLongReal(long double &x)
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'r') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    float f;
    fscanf(O2M_M_f, "%e", &f);
    x = f;
    return 1;
}

//---------------------------------------------------------------
//     
int FileIO::TFile::ReadName(char nme[])
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'r') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    fscanf(O2M_M_f, "%s", nme);
    return 1;
}

//---------------------------------------------------------------
//    
int FileIO::TFile::ReadString(char *str, int str_len)
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'r') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    fscanf(O2M_M_f, "%s", str);
    return 1;
}         

//---------------------------------------------------------------
//    
int FileIO::TFile::WriteChar(char ch)
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'w') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    fprintf(O2M_M_f, "%c", ch);
    return 1;
}

//---------------------------------------------------------------
//     
int FileIO::TFile::WriteString(char *str, int str_size)
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'w') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    for(int i = 0; i < str_size, str[i] != '\0'; i++)
        fprintf(O2M_M_f, "%c", str[i]);
    return 1;
}

//---------------------------------------------------------------
//     
int FileIO::TFile::WriteInt(long x, long n)
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'w') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    fprintf(O2M_M_f, "%i", x);
    return 1;
}

//---------------------------------------------------------------
//     
int FileIO::TFile::WriteReal(double x, long n)
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'w') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    fprintf(O2M_M_f, "%e", x);
    return 1;
}

//---------------------------------------------------------------
//      
int FileIO::TFile::WriteLongReal(long double x, long n)
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'w') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    fprintf(O2M_M_f, "%e", x);
    return 1;
}

//---------------------------------------------------------------
//      
int FileIO::TFile::Ln()
{
    if (O2M_M_IORes) return 0;
    if (O2M_M_OpenState != 'w') {
        O2M_M_IORes = NotOpenedForWriting;
        return 0;
    }
    fprintf(O2M_M_f, "\n");
    return 1;
}


//---------------------------------------------------------------
//  
void FileIO::O2M_SYS_FileIO()
{}
