//////////////////////////////////////////
//      Windows //
//////////////////////////////////////////

#include "WinConsole.h"

#include <wincon.h>


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// 
static HANDLE O2M_Input;


//    
WinConsole::TScreenBuffer WinConsole::StdOutput(true);


//-----------------------------------------------------------------------------
//     
WinConsole::TScreenBuffer::TScreenBuffer() : O2M_M_Handle(INVALID_HANDLE_VALUE), Error(false) {
	O2M_M_Handle = CreateConsoleScreenBuffer(GENERIC_WRITE | GENERIC_READ, 0, 0, CONSOLE_TEXTMODE_BUFFER, 0);
	if (O2M_M_Handle == INVALID_HANDLE_VALUE) throw "ERROR: Cannot create WinConsole.ConsoleScreenBuffer";
}


//-----------------------------------------------------------------------------
//   ,   STD_OUTPUT_BUFFER
WinConsole::TScreenBuffer::TScreenBuffer(bool O2M_FAKE_PARAMETER) : O2M_M_Handle(INVALID_HANDLE_VALUE) {
	O2M_M_Handle = GetStdHandle(STD_OUTPUT_HANDLE);
	if (O2M_M_Handle == INVALID_HANDLE_VALUE) throw "ERROR: Cannot get WinConsole.StdOutput";
}


//-----------------------------------------------------------------------------
//  ,     Windows
int WinConsole::GetWinTickCount() {
	return GetTickCount();
}


//-----------------------------------------------------------------------------
//     (  )
void WinConsole::TScreenBuffer::SetActive() {
	Error = SetConsoleActiveScreenBuffer(O2M_M_Handle) != 0;
}


//-----------------------------------------------------------------------------
//  
void WinConsole::TScreenBuffer::SetCursorPos(const short X, const short Y) {
	COORD coord;
	coord.X = X;
	coord.Y = Y;
	Error = SetConsoleCursorPosition(O2M_M_Handle, coord) != 0;
}


//-----------------------------------------------------------------------------
// Count    ,     ,
// - -   )
int WinConsole::TScreenBuffer::Write(const char *S, const int SSize, const int Count) {
	DWORD num = Count > SSize ? SSize : Count;
	if (!WriteConsole(O2M_M_Handle, S, num, &num, 0)) return 0;
	return num;
}


//-----------------------------------------------------------------------------
//     ,     
// - -   )
void WinConsole::TScreenBuffer::WriteStr(const char *S, const int SSize) {
	int i = 0;
	while ((i < SSize) && (S[i] != 0)) ++i;
	DWORD num;
	if (!WriteConsole(O2M_M_Handle, S, i, &num, 0))
		Error = true;
	else
		Error = false;
}


//-----------------------------------------------------------------------------
// Count      ,
// -    
int WinConsole::TScreenBuffer::ReadScreen (char *S, const int SSize, const short X, const short Y, const int Count) {
	DWORD num = Count > SSize ? SSize : Count;
	COORD coord;
	coord.X = X;
	coord.Y = Y;
	if (!ReadConsoleOutputCharacter(O2M_M_Handle, S, num, coord, &num)) return 0;
	return num;
}


//-----------------------------------------------------------------------------
//    
void WinConsole::TScreenBuffer::SetTextAttr(const short TextColor, const short BackgroundColor) {
	Error = SetConsoleTextAttribute(O2M_M_Handle, BackgroundColor * 16 + TextColor) != 0;
}


//-----------------------------------------------------------------------------
//       ,
// -      
int WinConsole::TScreenBuffer::FillAttr(const short TextColor, const short BackgroundColor, const short X, const short Y, const int Count) {
	COORD coord;
	coord.X = X;
	coord.Y = Y;
	DWORD num;
	if (!FillConsoleOutputAttribute(O2M_M_Handle, BackgroundColor * 16 + TextColor, Count, coord, &num)) return 0;
	return num;
}


//-----------------------------------------------------------------------------
//  
void WinConsole::TScreenBuffer::GetCursorPos(short &X, short &Y) {
	CONSOLE_SCREEN_BUFFER_INFO SBInfo;
	if (!GetConsoleScreenBufferInfo(O2M_M_Handle, &SBInfo)) {
		Error = true;
		return;
	}
	X = SBInfo.dwCursorPosition.X;
	Y = SBInfo.dwCursorPosition.Y;
	Error = false;
}


//-----------------------------------------------------------------------------
// Count        ,
// -    
int WinConsole::Read (char *S, const int SSize, const int Count) {
	DWORD num = Count > SSize ? SSize : Count;
	if (!ReadConsole(O2M_Input, S, num, &num, 0)) return 0;
	return num;
}


//-----------------------------------------------------------------------------
//        
// 0     (,  )
char WinConsole::ReadChar() {
	INPUT_RECORD irec;
	DWORD num;
	if (!ReadConsoleInput(O2M_Input, &irec, 1, &num) || !num || !(irec.EventType & KEY_EVENT)) return 0;
	if (!irec.Event.KeyEvent.bKeyDown) return 0;
	return irec.Event.KeyEvent.uChar.AsciiChar;
}


//-----------------------------------------------------------------------------
//         
// 0     (,  )
//KeyDown   TRUE   ,  FALSE  
char WinConsole::ReadCharEx(bool &KeyDown) {
	INPUT_RECORD irec;
	DWORD num;
	if (!ReadConsoleInput(O2M_Input, &irec, 1, &num) || !num || !(irec.EventType & KEY_EVENT)) return 0;
	KeyDown = irec.Event.KeyEvent.bKeyDown != 0;
	return irec.Event.KeyEvent.uChar.AsciiChar;
}


//-----------------------------------------------------------------------------
//          
// 0     (,  )
short WinConsole::ReadKey() {
	INPUT_RECORD irec;
	DWORD num;
	if (!ReadConsoleInput(O2M_Input, &irec, 1, &num) || !num || !(irec.EventType & KEY_EVENT)) return 0;
	if (!irec.Event.KeyEvent.bKeyDown) return 0;
	return irec.Event.KeyEvent.wVirtualKeyCode;
}


//-----------------------------------------------------------------------------
//           
// 0     (,  )
//KeyDown   TRUE   ,  FALSE  
short WinConsole::ReadKeyEx(bool &KeyDown) {
	INPUT_RECORD irec;
	DWORD num;
	if (!ReadConsoleInput(O2M_Input, &irec, 1, &num) || !num || !(irec.EventType & KEY_EVENT)) return 0;
	KeyDown = irec.Event.KeyEvent.bKeyDown != 0;
	return irec.Event.KeyEvent.wVirtualKeyCode;
}


//-----------------------------------------------------------------------------
//  
void WinConsole::FlushInputBuffer() {
	FlushConsoleInputBuffer(O2M_Input);
}


//-----------------------------------------------------------------------------
//  ,
//EditStyleInput:
//  TRUE  -      ,  -  Enter
//  FALSE -         
//ProcessedInput:
//  TRUE  -   (Ctrl+C  ..)     
//  FALSE -   ( .. Ctrl+C  ..)    
//    SetInputMode(FALSE, TRUE)
void WinConsole::SetInputMode(const bool EditStyleInput, const bool ProcessedInput) {
	SetConsoleMode(O2M_Input, (EditStyleInput ? (ENABLE_LINE_INPUT | ENABLE_ECHO_INPUT) : 0) | (ProcessedInput ? ENABLE_PROCESSED_INPUT : 0));
}


//-----------------------------------------------------------------------------
//      -
// / ,    ..
bool WinConsole::InputBufferEmpty() {
	DWORD num;
	if (!GetNumberOfConsoleInputEvents(O2M_Input, &num)) return true;
	return num == 0;
}


//-----------------------------------------------------------------------------
//  
void WinConsole::SetTitle(const char *Title, const int TitleSize) {
	StdOutput.Error = SetConsoleTitle(Title) != 0;
}


//---------------------------------------------------------------
//  
void WinConsole::O2M_SYS_WinConsole() {
	O2M_Input = GetStdHandle(STD_INPUT_HANDLE);
	if (O2M_Input == INVALID_HANDLE_VALUE) throw ("ERROR: Cannot get WinConsole.O2M_Input");
	SetConsoleMode(O2M_Input, ENABLE_PROCESSED_INPUT);
}
