/////////////////////////////////////////////////////////
//      Windows //
//       //
// Win32 GDI                                           //
/////////////////////////////////////////////////////////

#include "WinApp.h"
#include "O2M_WinApp.h"

using namespace WinApp;

// ,   namespace WinApp

//  
HWND WinApp::hWnd = 0;
//    
HDC WinApp::hdc = 0;
//    
HDC WinApp::btmDC = 0;
//     (    )
RECT WinApp::ClientRect;
//  
POINT WinApp::CurrPos;
//  
LOGPEN WinApp::CurrPen;
//  
LOGBRUSH WinApp::CurrBrush;
//    
bool WinApp::KeyArray[KeyCount];
//    O2M
bool volatile WinApp::KillThread = false;


//-----------------------------------------------------------------------------
//     
static int ErrorBox(const char *st) {
	return MessageBox(NULL, st, "Error", MB_OK | MB_ICONERROR | MB_DEFBUTTON1);
}


//-----------------------------------------------------------------------------
// 
LONG WINAPI WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {
	// 
	case WM_KEYDOWN:
		{
			int k = int(wParam);
			if (0 < k && KeyCount > k) KeyArray[k] = true;
			return 0;
		}
	// 
	case WM_KEYUP:
		{
			int k = int(wParam);
			if (0 < k && KeyCount > k) KeyArray[k] = false;
			return 0;
		}
	//  
	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;
	//  
	case WM_PAINT:
		{
			//     
			RECT r;
			if (!GetUpdateRect(hWnd, &r, false)) break;
			//    
			PAINTSTRUCT pnt;
			if (!BeginPaint(hWnd, &pnt)) break;
			//         ,  
			BitBlt(hdc, r.left, r.top, r.right - r.left, r.bottom - r.top, btmDC, r.left, r.top, SRCCOPY);
			//   
			EndPaint(hWnd, &pnt);
			return 0;
		}
	}//switch

	//      
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}


//-----------------------------------------------------------------------------
//       O2M
void __cdecl ThreadProc(void *ThreadData)
{
	//    
	O2M_SYS_main_init();
}


//-----------------------------------------------------------------------------
//   
int APIENTRY WinMain(HINSTANCE hInst, HINSTANCE hPreviousInst,
	LPSTR lpCmdLine, int nCmdShow)
{
	//   
	LPCTSTR lpszClassName = "WinAppClass";
	//     
	WNDCLASS wc;
	//     
	const int xPos = 100, yPos = 100, xSize = 600, ySize = 400;

	//     
	wc.style = CS_GLOBALCLASS | CS_PARENTDC;
	wc.lpfnWndProc = WindowProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = 0;
	wc.hIcon = LoadIcon(0, IDI_ASTERISK);
	wc.hCursor = LoadCursor(0, IDC_ARROW);
	wc.hbrBackground = CreateSolidBrush(0);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = lpszClassName;

	//    
	ATOM at = RegisterClass(&wc);
	if (!at) {
		ErrorBox("RegisterClass failed");
		return 1;
	}

	//   
	hWnd = CreateWindow(lpszClassName, "O2M Application", WS_OVERLAPPED | WS_VISIBLE | WS_SYSMENU | WS_MINIMIZEBOX, xPos, yPos, xSize, ySize, NULL, NULL, 0, NULL);
	if (!hWnd) {
		ErrorBox("CreateWindow failed");
		return 1;
	}

	//      ( )
	if (!GetClientRect(hWnd, &ClientRect)) {
		ErrorBox("GetClientRect failed");
		return 1;
	}

	//   
	hdc = GetDC(hWnd);
	if (!hdc) {
		ErrorBox("GetDC failed");
		return 1;
	}
	//     
	btmDC = CreateCompatibleDC(hdc);
	if (!btmDC) {
		ErrorBox("CreateCompatibleDC failed");
		return 1;
	}

	//      
	HBITMAP btm = CreateCompatibleBitmap(hdc, ClientRect.right, ClientRect.bottom);
	if (!btm) {
		ErrorBox("CreateCompatibleBitmap failed");
		return 1;
	}
	//    btmDC
	btm = (HBITMAP)SelectObject(btmDC, btm);
	if (!btm) {
		ErrorBox("Bitmap Selection failed");
		return 1;
	}

	//   
	CurrPen.lopnStyle = PS_SOLID;
	CurrPen.lopnWidth.x = 0;
	CurrPen.lopnColor = 0xFFFFFF;
	//     
	HPEN hpen = CreatePenIndirect(&CurrPen);
	if (!hpen) {
		ErrorBox("CreatePenIndirect failed");
		return 1;
	}
	//        
	hpen = (HPEN)SelectObject(btmDC, hpen);
	if (!hpen) {
		ErrorBox("Pen Selection failed");
		return 1;
	}

	//   
	CurrBrush.lbStyle = BS_SOLID;
	CurrBrush.lbColor = 0;
	CurrBrush.lbHatch = HS_CROSS;
	//     
	HBRUSH hbrush = CreateBrushIndirect(&CurrBrush);
	if (!hbrush) {
		ErrorBox("CreateBrushIndirect failed");
		return 1;
	}
	//        
	hbrush = (HBRUSH)SelectObject(btmDC, hbrush);
	if (!hbrush) {
		ErrorBox("Brush Selection failed");
		return 1;
	}

	//    
	SetBkColor(btmDC, 0);
	//  
	MoveToEx(btmDC, 0, 0, 0);
	CurrPos.x = 0;
	CurrPos.y = 0;

	//   2
	_beginthread(ThreadProc, 0, 0);

	//    
	MSG msg;
	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	//    2
	KillThread = true;

	//  
	hbrush = (HBRUSH)SelectObject(btmDC, hbrush);
	DeleteObject(hbrush);
	//  
	hpen = (HPEN)SelectObject(btmDC, hpen);
	DeleteObject(hpen);
	// 
	btm = (HBITMAP)SelectObject(btmDC, btm);
	DeleteObject(btm);
	//     
	DeleteDC(btmDC);
	//   
	ReleaseDC(hWnd, hdc);

	// 
	DestroyWindow(hWnd);

	return msg.wParam;
}


//-----------------------------------------------------------------------------
//   WinApp
void WinApp::O2M_SYS_WinApp()
{}
