//////////////////////////////////////////////////
//       Win32 GDI //
//////////////////////////////////////////////////

#include "WinGraph.h"
#include "O2M_WinApp.h"

using namespace WinApp;

// ,   namespace WinGraph

//  
const int WinGraph::fsBorder = FLOODFILLBORDER;		// ,   
const int WinGraph::fsSurface = FLOODFILLSURFACE;	// ,   
//  
const int WinGraph::psSolid = PS_SOLID;				// 
const int WinGraph::psDash = PS_DASH;				// 
const int WinGraph::psDot = PS_DOT;					//  
const int WinGraph::psDashDot = PS_DASHDOT;			//    
const int WinGraph::psDashDotDot = PS_DASHDOTDOT;	//     
const int WinGraph::psClear = PS_NULL;				// 
//      
const int WinGraph::bkOpaque = OPAQUE;				//   
const int WinGraph::bkTransparent = TRANSPARENT;	//  


//-----------------------------------------------------------------------------
// ,    
void WinGraph::Arc (int LeftRect, int TopRect, int RightRect, int BottomRect, int XStartArc, int YStartArc, int XEndArc, int YEndArc) {
	ASSERT_GRAPH_MODE
	//  Win32 GUI
	if (!::Arc(btmDC, LeftRect, TopRect, RightRect, BottomRect, XStartArc, YStartArc, XEndArc, YEndArc)) return;
	//   ,  
	RECT r;
	r.left = LeftRect;
	r.top = TopRect;
	r.right = RightRect;
	r.bottom = BottomRect;
	InvalidateRect(hWnd, &r, false);
}


//-----------------------------------------------------------------------------
//      
void WinGraph::Clear () {
	ASSERT_GRAPH_MODE
	//     
	HBRUSH hbrush = ::CreateSolidBrush(::GetBkColor(btmDC));
	if (!hbrush) return;
	//  
	::FillRect(btmDC, &ClientRect, hbrush);
	//   
	DeleteObject(hbrush);
	//     
	InvalidateRect(hWnd, NULL, false);
}


//-----------------------------------------------------------------------------
//     
void WinGraph::Ellipse (int LeftRect, int TopRect, int RightRect, int BottomRect) {
	ASSERT_GRAPH_MODE
	//  Win32 GUI
	if (!::Ellipse(btmDC, LeftRect, TopRect, RightRect, BottomRect)) return;
	//   ,  
	RECT r;
	r.left = LeftRect;
	r.top = TopRect;
	r.right = RightRect;
	r.bottom = BottomRect;
	InvalidateRect(hWnd, &r, false);
}


//-----------------------------------------------------------------------------
//  ,    
void WinGraph::FloodFill (int x, int y, TColor Color, TFillStyle FillStyle) {
	ASSERT_GRAPH_MODE
	//  Win32 GUI
	::ExtFloodFill(btmDC, x, y, Color, FillStyle);
	//     
	InvalidateRect(hWnd, NULL, false);
}


//-----------------------------------------------------------------------------
//    (x1, y1)  (x2, y2)
void WinGraph::Line (int x1, int y1, int x2, int y2) {
	ASSERT_GRAPH_MODE
	// 
	if (!::MoveToEx(btmDC, x1, y1, 0)) return;
	::LineTo(btmDC, x2, y2);
	//  
	::MoveToEx(btmDC, CurrPos.x, CurrPos.y, 0);
	//   ,  
	RECT r;
	if (x1 < x2) {
		r.left = x1;
		r.right = x2 + 1;
	} else {
		r.left = x2;
		r.right = x1 + 1;
	}
	if (y1 < y2) {
		r.top = y1;
		r.bottom = y2 + 1;
	} else {
		r.top = y2;
		r.bottom = y1 + 1;
	}
	InvalidateRect(hWnd, &r, false);
}


//-----------------------------------------------------------------------------
//       
void WinGraph::LineTo (int x, int y) {
	ASSERT_GRAPH_MODE
	//  Win32 GUI
	if (!::LineTo(btmDC, x, y)) return;
	//   ,  
	RECT r;
	if (CurrPos.x > x) {
		r.left = x;
		r.right = CurrPos.x + 1;
	} else {
		r.left = CurrPos.x;
		r.right = x + 1;
	}
	if (CurrPos.y > y) {
		r.top = y;
		r.bottom = CurrPos.y + 1;
	} else {
		r.top = CurrPos.y;
		r.bottom = y + 1;
	}
	InvalidateRect(hWnd, &r, false);
	//    
	CurrPos.x = x;
	CurrPos.y = y;
}


//-----------------------------------------------------------------------------
//     
void WinGraph::MoveTo(int x, int y) {
	ASSERT_GRAPH_MODE
	//  Win32 GUI
	if (::MoveToEx(btmDC, x, y, 0)) {
		CurrPos.x = x;
		CurrPos.y = y;
	}
}


//-----------------------------------------------------------------------------
//      
void WinGraph::OutString (int x, int y, char *O2M_ARR_String, const int O2M_ARR_0_String) {
	OutText(x, y, O2M_ARR_String, O2M_ARR_0_String, O2M_ARR_0_String - 1);
}


//-----------------------------------------------------------------------------
//       
void WinGraph::OutText (int x, int y, char *O2M_ARR_Text, const int O2M_ARR_0_Text, int CharCount) {
	ASSERT_GRAPH_MODE
	//    
	if (O2M_ARR_0_Text < CharCount) CharCount = O2M_ARR_0_Text;
	//  Win32 GUI
	::TextOut(btmDC, x, y, O2M_ARR_Text, CharCount);
	//   ,  
	InvalidateRect(hWnd, NULL, false);
}


//-----------------------------------------------------------------------------
// ,    
void WinGraph::Pie (int LeftRect, int TopRect, int RightRect, int BottomRect, int XRadial1, int YRadial1, int XRadial2, int YRadial2) {
	ASSERT_GRAPH_MODE
	//  Win32 GUI
	if (!::Pie(btmDC, LeftRect, TopRect, RightRect, BottomRect, XRadial1, YRadial1, XRadial2, YRadial2)) return;
	//   ,  
	RECT r;
	r.left = LeftRect;
	r.top = TopRect;
	r.right = RightRect;
	r.bottom = BottomRect;
	InvalidateRect(hWnd, &r, false);
}


//-----------------------------------------------------------------------------
//      
void WinGraph::PieSlice (int x, int y, int Radius, double StartAngle, double SweepAngle) {
	ASSERT_GRAPH_MODE
	//     
	::MoveToEx(btmDC, x, y, 0);
	//    
	::BeginPath(btmDC);
	::AngleArc(btmDC, x, y, Radius, (float)StartAngle, (float)SweepAngle);
	::LineTo(btmDC, x, y);
	::EndPath(btmDC);
	::StrokeAndFillPath(btmDC);
	//  
	::MoveToEx(btmDC, CurrPos.x, CurrPos.y, 0);
	//   ,  
	RECT r;
	r.left = x - Radius;
	r.top = y - Radius;
	r.right = x + Radius + 1;
	r.bottom = y + Radius + 1;
	InvalidateRect(hWnd, &r, false);
}


//-----------------------------------------------------------------------------
//   
void WinGraph::Rectangle (int LeftRect, int TopRect, int RightRect, int BottomRect) {
	ASSERT_GRAPH_MODE
	//  Win32 GUI
	if (!::Rectangle(btmDC, LeftRect, TopRect, RightRect, BottomRect)) return;
	//   ,  
	RECT r;
	r.left = LeftRect;
	r.top = TopRect;
	r.right = RightRect;
	r.bottom = BottomRect;
	InvalidateRect(hWnd, &r, false);
}


//-----------------------------------------------------------------------------
//   
void WinGraph::RoundRect (int LeftRect, int TopRect, int RightRect, int BottomRect, int WidthEllipse, int HeightEllipse) {
	ASSERT_GRAPH_MODE
	//  Win32 GUI
	if (!::RoundRect(btmDC, LeftRect, TopRect, RightRect, BottomRect, WidthEllipse, HeightEllipse)) return;
	//   ,  
	RECT r;
	r.left = LeftRect;
	r.top = TopRect;
	r.right = RightRect;
	r.bottom = BottomRect;
	InvalidateRect(hWnd, &r, false);
}


//-----------------------------------------------------------------------------
//   
void WinGraph::SetBkColor (TColor Color) {
	ASSERT_GRAPH_MODE
	//  Win32 GUI
	::SetBkColor(btmDC, Color);
}


//-----------------------------------------------------------------------------
//       
void WinGraph::SetBkMode (TBkMode Mode) {
	ASSERT_GRAPH_MODE
	//  Win32 GUI
	::SetBkMode(btmDC, Mode);
}


//-----------------------------------------------------------------------------
//   
void WinGraph::SetBrushColor (TColor Color) {
	ASSERT_GRAPH_MODE
	//    
	CurrBrush.lbColor = Color;
	HBRUSH hbrush = ::CreateBrushIndirect(&CurrBrush);
	if (!hbrush) return;
	//      
	hbrush = (HBRUSH)SelectObject(btmDC, hbrush);
	//  
	DeleteObject(hbrush);
}


//-----------------------------------------------------------------------------
//   
void WinGraph::SetBrushStyle (TBrushStyle BrushStyle) {
	ASSERT_GRAPH_MODE
	//        
	switch (BrushStyle) {
	case bsSolid:
		CurrBrush.lbStyle = BS_SOLID;
		break;
	case bsClear:
		CurrBrush.lbStyle = BS_NULL;
		break;
	case bsBDiagonal:
		CurrBrush.lbStyle = BS_HATCHED;
		CurrBrush.lbHatch = HS_BDIAGONAL;
		break;
	case bsCross:
		CurrBrush.lbStyle = BS_HATCHED;
		CurrBrush.lbHatch = HS_CROSS;
		break;
	case bsDiagCross:
		CurrBrush.lbStyle = BS_HATCHED;
		CurrBrush.lbHatch = HS_DIAGCROSS;
		break;
	case bsFDiagonal:
		CurrBrush.lbStyle = BS_HATCHED;
		CurrBrush.lbHatch = HS_FDIAGONAL;
		break;
	case bsHorizontal:
		CurrBrush.lbStyle = BS_HATCHED;
		CurrBrush.lbHatch = HS_HORIZONTAL;
		break;
	case bsVertical:
		CurrBrush.lbStyle = BS_HATCHED;
		CurrBrush.lbHatch = HS_VERTICAL;
		break;
	default:
		return;
	}//switch
	//    
	HBRUSH hbrush = ::CreateBrushIndirect(&CurrBrush);
	if (!hbrush) return;
	//      
	hbrush = (HBRUSH)SelectObject(btmDC, hbrush);
	//  
	DeleteObject(hbrush);
}


//-----------------------------------------------------------------------------
//   
void WinGraph::SetPenColor(TColor Color) {
	ASSERT_GRAPH_MODE
	//    
	CurrPen.lopnColor = Color;
	HPEN hpen = ::CreatePenIndirect(&CurrPen);
	if (!hpen) return;
	//      
	hpen = (HPEN)SelectObject(btmDC, hpen);
	//  
	DeleteObject(hpen);
}


//-----------------------------------------------------------------------------
//   
void WinGraph::SetPenStyle (TPenStyle Style) {
	ASSERT_GRAPH_MODE
	//    
	CurrPen.lopnStyle = Style;
	HPEN hpen = ::CreatePenIndirect(&CurrPen);
	if (!hpen) return;
	//      
	hpen = (HPEN)SelectObject(btmDC, hpen);
	//  
	DeleteObject(hpen);
}


//-----------------------------------------------------------------------------
//       
void WinGraph::SetPixel (int x, int y, TColor Color) {
	ASSERT_GRAPH_MODE
	//  Win32 GUI
	::SetPixel(btmDC, x, y, Color);
	//   ,  
	RECT r;
	r.left = x;
	r.top = y;
	r.right = x + 1;
	r.bottom = y + 1;
	InvalidateRect(hWnd, &r, false);
}


//-----------------------------------------------------------------------------
//   
void WinGraph::SetTextColor (TColor Color) {
	ASSERT_GRAPH_MODE
	//  Win32 GUI
	::SetTextColor(btmDC, Color);
}


//-----------------------------------------------------------------------------
//   
int WinGraph::TextHeight (char *O2M_ARR_Text, const int O2M_ARR_0_Text, int CharCount) {
	ASSERT_GRAPH_MODE
	//    
	if (O2M_ARR_0_Text < CharCount) CharCount = O2M_ARR_0_Text;
	//  Win32 GUI
	SIZE sz;
	if (::GetTextExtentPoint32(btmDC, O2M_ARR_Text, CharCount, &sz))
		return sz.cy;
	else
		return 0;
}


//-----------------------------------------------------------------------------
//   
int WinGraph::TextWidth (char *O2M_ARR_Text, const int O2M_ARR_0_Text, int CharCount) {
	ASSERT_GRAPH_MODE
	//    
	if (O2M_ARR_0_Text < CharCount) CharCount = O2M_ARR_0_Text;
	//  Win32 GUI
	SIZE sz;
	if (::GetTextExtentPoint32(btmDC, O2M_ARR_Text, CharCount, &sz))
		return sz.cx;
	else
		return 0;
}


//-----------------------------------------------------------------------------
//   WinGraph
void WinGraph::O2M_SYS_WinGraph()
{}
